@@ -196,7 +196,7 @@ class Converter(object):
         if output_tags is not None:
             for tag in output_tags:
                 if tag not in self._schema:
-                    logger.warning(
+                    logger.debug(
                         'Specified tag "{tag}" not found in config schema: '
                         'available options are {schema_keys}'.format(
                             tag=tag, schema_keys=str(list(self._schema.keys()))))
@@ -399,6 +399,9 @@ class Converter(object):
             for name, value in item['output'].items():
                 pretty_value = self._prettify(value)
                 record[name] = pretty_value if isinstance(pretty_value, str) else json.dumps(pretty_value, ensure_ascii=False)
+            for name, value in item['input'].items():
+                if isinstance(value, dict) or isinstance(value, list):
+                    record[name] = json.dumps(value, ensure_ascii=False)
             record['annotator'] = _get_annotator(item)
             record['annotation_id'] = item['annotation_id']
             record['created_at'] = item['created_at']
@@ -408,7 +411,26 @@ class Converter(object):
                 record['agreement'] = item['agreement']
             records.append(record)
 
-        pd.DataFrame.from_records(records).to_csv(output_file, index=False, **kwargs)
+        # Previously we were using pandas dataframe to_csv() but that produced incorrect JSON so writing manually
+        with open(output_file, 'w') as outfile:
+            if kwargs['header']:
+                keys = records[0].keys()
+                outfile.write(kwargs['sep'].join(keys) + '\n')
+            for record in records:
+                line = []
+                for key in keys:
+                    if record[key] is None:
+                        line.append('')
+                    elif key == 'annotation_id':
+                        # Replicating previous implementation of converting None values to pandas.NA
+                        # which outputs in CSV files as an empty string
+                        if record[key] == None:
+                            line.append('')
+                        else:
+                            line.append(str(record[key]))
+                    else:
+                        line.append(str(record[key]))
+                outfile.write(kwargs['sep'].join(line) + '\n')
 
     def convert_to_conll2003(self, input_data, output_dir, is_dir=True):
         self._check_format(Format.CONLL2003)
@@ -464,7 +486,7 @@ class Converter(object):
                                           return_relative_path=True, upload_dir=self.upload_dir,
                                           download_resources=self.download_resources)
                 except:
-                    logger.error('Unable to download {image_path}. The image of {item} will be skipped'.format(
+                    logger.info('Unable to download {image_path}. The image of {item} will be skipped'.format(
                         image_path=image_path, item=item
                     ), exc_info=True)
             # add image to final images list
@@ -473,7 +495,7 @@ class Converter(object):
                     width, height = img.size
                 images = add_image(images, width, height, image_id, image_path)
             except:
-                logger.error("Unable to open {image_path}, can't extract width and height for COCO export".format(
+                logger.info("Unable to open {image_path}, can't extract width and height for COCO export".format(
                     image_path=image_path, item=item
                 ), exc_info=True)
 
@@ -486,13 +508,13 @@ class Converter(object):
                 logger.warning('No annotations found for item #' + str(item_idx))
                 continue
           
-            # concatentate results over all tag names
+            # concatenate results over all tag names
             labels = []
             for key in item['output']:
                 labels += item['output'][key]
 
             if len(labels) == 0:
-                logger.warning(f'Empty bboxes for {item["output"]}')
+                logger.debug(f'Empty bboxes for {item["output"]}')
                 continue
 
             for label in labels:
@@ -504,12 +526,12 @@ class Converter(object):
                         break
 
                 if category_name is None:
-                    logger.warning("Unknown label type or labels are empty: " + str(label))
+                    logger.warning("Unknown label type or labels are empty")
                     continue
 
                 if not height or not width:
                     if 'original_width' not in label or 'original_height' not in label:
-                        logger.warning(f'original_width or original_height not found in {image_path}')
+                        logger.debug(f'original_width or original_height not found in {image_path}')
                         continue
 
                     width, height = label['original_width'], label['original_height']
@@ -520,10 +542,12 @@ class Converter(object):
                 annotation_id = len(annotations)
 
                 if 'rectanglelabels' in label or 'labels' in label:
-                    x = int(label['x'] / 100 * width)
-                    y = int(label['y'] / 100 * height)
-                    w = int(label['width'] / 100 * width)
-                    h = int(label['height'] / 100 * height)
+                    x, y, w, h = self.rotated_rectangle(label)
+                    
+                    x = x * label["original_width"] / 100
+                    y = y * label["original_height"] / 100
+                    w = w * label["original_width"] / 100
+                    h = h * label["original_height"] / 100
 
                     annotations.append({
                         'id': annotation_id,
@@ -589,19 +613,27 @@ class Converter(object):
         data_key = self._data_keys[0]
         item_iterator = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
         for item_idx, item in enumerate(item_iterator):
-            if not item['output']:
-                logger.warning('No completions found for item #' + str(item_idx))
-                continue
+            # get image path and label file path
             image_path = item['input'][data_key]
+            # Download image
             if not os.path.exists(image_path):
                 try:
                     image_path = download(image_path, output_image_dir, project_dir=self.project_dir,
                                           return_relative_path=True, upload_dir=self.upload_dir,
                                           download_resources=self.download_resources)
                 except:
-                    logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
+                    logger.info('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item
                     ), exc_info=True)
+            # identify label file path
+            label_path = os.path.join(output_label_dir, os.path.splitext(os.path.basename(image_path))[0] + '.txt')
+            # Skip tasks without annotations
+            if not item['output']:
+                logger.warning('No completions found for item #' + str(item_idx))
+                if not os.path.exists(label_path):
+                    with open(label_path, 'x'):
+                        pass
+                continue
 
             # concatenate results over all tag names
             labels = []
@@ -610,9 +642,11 @@ class Converter(object):
 
             if len(labels) == 0:
                 logger.warning(f'Empty bboxes for {item["output"]}')
+                if not os.path.exists(label_path):
+                    with open(label_path, 'x'):
+                        pass
                 continue
 
-            label_path = os.path.join(output_label_dir, os.path.splitext(os.path.basename(image_path))[0]+'.txt')
             annotations = []
             for label in labels:
                 category_name = None
@@ -624,7 +658,7 @@ class Converter(object):
                             category_names.append(item)
 
                 if len(category_names) == 0:
-                    logger.warning("Unknown label type or labels are empty: " + str(label))
+                    logger.debug("Unknown label type or labels are empty: " + str(label))
                     continue
 
                 for category_name in category_names:
@@ -638,43 +672,8 @@ class Converter(object):
                     category_id = category_name_to_id[category_name]
 
                     if "rectanglelabels" in label or 'labels' in label:
-                        label_x, label_y, label_w, label_h, label_r = (
-                            label["x"],
-                            label["y"],
-                            label["width"],
-                            label["height"],
-                            label["rotation"],
-                        )
-                        if abs(label_r) > 0:
-                            r = math.pi * label_r / 180
-                            sin_r = math.sin(r)
-                            cos_r = math.cos(r)
-                            h_sin_r, h_cos_r = label_h * sin_r, label_h * cos_r
-                            x_top_right = label_x + label_w * cos_r
-                            y_top_right = label_y + label_w * sin_r
-
-                            x_ls = [
-                                label_x,
-                                x_top_right,
-                                x_top_right - h_sin_r,
-                                label_x - h_sin_r,
-                            ]
-                            y_ls = [
-                                label_y,
-                                y_top_right,
-                                y_top_right + h_cos_r,
-                                label_y + h_cos_r,
-                            ]
-                            label_x = max(0, min(x_ls))
-                            label_y = max(0, min(y_ls))
-                            label_w = min(100, max(x_ls)) - label_x
-                            label_h = min(100, max(y_ls)) - label_y
-
-                        x = (label_x + label_w / 2) / 100
-                        y = (label_y + label_h / 2) / 100
-                        w = label_w / 100
-                        h = label_h / 100
-                        annotations.append([category_id, x, y, w, h])
+                        x, y, w, h = self.rotated_rectangle(label)
+                        annotations.append([category_id, (x + w / 2) / 100, (y + h / 2) / 100, w / 100, h / 100])
                     else:
                         raise ValueError(f"Unknown label type {label}")
             with open(label_path, 'w') as f:
@@ -697,6 +696,51 @@ class Converter(object):
                 }
             }, fout, indent=2)
 
+    @staticmethod
+    def rotated_rectangle(label):
+        label_x, label_y, label_w, label_h, label_r = (
+            label["x"],
+            label["y"],
+            label["width"],
+            label["height"],
+            label["rotation"],
+        )
+        
+        if abs(label_r) > 0:
+            alpha = math.atan(label_h / label_w)
+            beta = math.pi * (label_r / 180)  # Label studio defines the angle towards the vertical axis
+            
+            radius = math.sqrt((label_w/2) ** 2 + (label_h/2) ** 2)
+            
+            # Label studio saves the position of top left corner after rotation
+            x_0 = label_x - radius * (math.cos(math.pi - alpha - beta) - math.cos(math.pi - alpha)) + label_w / 2
+            y_0 = label_y + radius * (math.sin(math.pi - alpha - beta) - math.sin(math.pi - alpha)) + label_h / 2
+            
+            theta_1 = alpha + beta
+            theta_2 = math.pi - alpha + beta
+            theta_3 = math.pi + alpha + beta
+            theta_4 = 2 * math.pi - alpha + beta
+
+            x_coord = [
+                x_0 + radius * math.cos(theta_1),
+                x_0 + radius * math.cos(theta_2),
+                x_0 + radius * math.cos(theta_3),
+                x_0 + radius * math.cos(theta_4),
+            ]
+            y_coord = [
+                y_0 + radius * math.sin(theta_1),
+                y_0 + radius * math.sin(theta_2),
+                y_0 + radius * math.sin(theta_3),
+                y_0 + radius * math.sin(theta_4),
+            ]
+            
+            label_x = min(x_coord)
+            label_y = min(y_coord)
+            label_w = max(x_coord) - label_x
+            label_h = max(y_coord) - label_y 
+        
+        return label_x, label_y, label_w, label_h
+
     def convert_to_voc(self, input_data, output_dir, output_image_dir=None, is_dir=True):
 
         ensure_dir(output_dir)
@@ -717,14 +761,11 @@ class Converter(object):
         data_key = self._data_keys[0]
         item_iterator = self.iter_from_dir(input_data) if is_dir else self.iter_from_json_file(input_data)
         for item_idx, item in enumerate(item_iterator):
-            if not item['output']:
-                logger.warning('No annotations found for item #' + str(item_idx))
-                continue
             image_path = item['input'][data_key]
             annotations_dir = os.path.join(output_dir, 'Annotations')
             if not os.path.exists(annotations_dir):
                 os.makedirs(annotations_dir)
-
+            # Download image
             channels = 3
             if not os.path.exists(image_path):
                 try:
@@ -732,7 +773,7 @@ class Converter(object):
                         image_path, output_image_dir, project_dir=self.project_dir,
                         upload_dir=self.upload_dir, return_relative_path=True, download_resources=self.download_resources)
                 except:
-                    logger.error('Unable to download {image_path}. The item {item} will be skipped'.format(
+                    logger.info('Unable to download {image_path}. The item {item} will be skipped'.format(
                         image_path=image_path, item=item), exc_info=True)
                 else:
                     full_image_path = os.path.join(output_image_dir, os.path.basename(image_path))
@@ -740,9 +781,15 @@ class Converter(object):
                     try:
                         _, _, channels = get_image_size_and_channels(full_image_path)
                     except:
-                        logger.warning(f"Can't read channels from image {image_path}")
+                        logger.warning(f"Can't read channels from image")
 
-            image_name = os.path.splitext(os.path.basename(image_path))[0]
+            # skip tasks without annotations
+            if not item['output']:
+                logger.warning('No annotations found for item #' + str(item_idx))
+                continue
+
+            image_name = os.path.basename(image_path)
+            xml_name = os.path.splitext(image_name)[0] + '.xml'
 
             # concatenate results over all tag names
             bboxes = []
@@ -750,15 +797,15 @@ class Converter(object):
                 bboxes += item['output'][key]
 
             if len(bboxes) == 0:
-                logger.warning(f'Empty bboxes for {item["output"]}')
+                logger.debug(f'Empty bboxes for {item["output"]}')
                 continue
 
             if 'original_width' not in bboxes[0] or 'original_height' not in bboxes[0]:
-                logger.warning(f'original_width or original_height not found in {image_name}')
+                logger.debug(f'original_width or original_height not found in {image_name}')
                 continue
 
             width, height = bboxes[0]['original_width'], bboxes[0]['original_height']
-            xml_filepath = os.path.join(annotations_dir, image_name + '.xml')
+            xml_filepath = os.path.join(annotations_dir, xml_name)
 
             my_dom = xml.dom.getDOMImplementation()
             doc = my_dom.createDocument(None, 'annotation', None)
